
#ifndef __ALGORITHMAN__CONTEXTFREEGRAMMAR_TPP
#define __ALGORITHMAN__CONTEXTFREEGRAMMAR_TPP

template<class token> ContextFreeGrammars::Terminal<token>::Terminal(std::string Name, token Token)
    : Symbol(Name)
{
    this->Token = Token;
}

template<class token> ContextFreeGrammars::Terminal<token>::~Terminal()
{
}




template<class token, int k> ContextFreeGrammars::Tokenizer<token,k>::Tokenizer(std::istream& input)
    : is(input)
{
    //this->is = is;
    lookaheadFoot = 0;
    for(int i = 0; i < k; i++)
    {
        is >> lookahead[lookaheadFoot];
        if(is.good())
            lookaheadFoot++;
        else
            break;
    }
}

template<class token, int k> token ContextFreeGrammars::Tokenizer<token,k>::ConsumeToken()
{
    token result = PeekToken(0);
    for(int i = 1; i < lookaheadFoot; i++)
        lookahead[i-1] = lookahead[i];

    lookaheadFoot--;
    is >> lookahead[lookaheadFoot];
    lookaheadFoot += is.good();

    return result;
}

template<class token, int k> bool ContextFreeGrammars::Tokenizer<token,k>::HasToken(int idx) const
{
    return idx < lookaheadFoot;
}

template<class token, int k> token ContextFreeGrammars::Tokenizer<token,k>::PeekToken(int idx) const
{
    if(idx >= lookaheadFoot)
        throw "Index out of bounds";

    return lookahead[idx];
}











template<class token> ContextFreeGrammars::ContextFreeGrammar<token>::ContextFreeGrammar()
{
}

template<class token> ContextFreeGrammars::ContextFreeGrammar<token>::ContextFreeGrammar(const ContextFreeGrammar& cfg)
{
    operator=(cfg);
}

template<class token> ContextFreeGrammars::ContextFreeGrammar<token>::~ContextFreeGrammar()
{
}

template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::Clear()
{
    for(typename std::vector<Terminal<token>*>::iterator i = terminals.begin(); i != terminals.end(); i++)
        delete *i;
    terminals.clear();
    for(std::vector<NonTerminal*>::iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
        delete *i;
    nonterminals.clear();

    symbols.clear();
}

template<class token> ContextFreeGrammars::ContextFreeGrammar<token>& ContextFreeGrammars::ContextFreeGrammar<token>::operator=(const ContextFreeGrammar<token>& cfg)
{
    Clear();

    // copy terminals
    for(typename std::vector<Terminal<token>*>::const_iterator i = cfg.terminals.begin(); i != cfg.terminals.end(); i++)
        AddTerminal((*i)->Token, (*i)->Name);

    // copy nonterminals
    for(std::vector<NonTerminal*>::const_iterator i = cfg.nonterminals.begin(); i != cfg.nonterminals.end(); i++)
        AddNonTerminal((*i)->Name);

    // copy productions
    for(std::vector<NonTerminal*>::const_iterator i = cfg.nonterminals.begin(); i != cfg.nonterminals.end(); i++)
    {
        NonTerminal* n = static_cast<NonTerminal*>(symbols[(*i)->Name]);
        for(std::vector<Production*>::iterator p = (*i)->productions.begin(); p != (*i)->productions.end(); p++)
        {
            Production* np = new Production();
            for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                np->produced.push_back(symbols[(*s)->Name]);
            n->AddOrDeleteProduction(np);
        }
    }

    // set start symbol
    startsymbol = static_cast<NonTerminal*>(symbols[cfg.startsymbol->Name]);

    return *this;
}






template<class token> bool ContextFreeGrammars::ContextFreeGrammar<token>::LoadFromStream(std::istream& is)
{
    Clear();

    std::string input;
    Symbol* symbol = NULL;
    NonTerminal* nonterminal = NULL;
    Production* production = NULL;

    for(is >> input; input != "%%"; is >> input)
    {
        token inputtoken;
        std::stringstream s;
        s << input;
        s >> inputtoken;
        AddTerminal(inputtoken, input);
    }
    
    reading_nonterminal:
        is >> input;
        if(input == "%%")
            goto finished;
        if(symbols.find(input) == symbols.end())
            AddNonTerminal(input);
        symbol = symbols[input];
        nonterminal = dynamic_cast<NonTerminal*>(symbol);
        if(nonterminal == NULL)
        {
            std::cerr << "Symbol " << input << " already defined as terminal." << std::endl;
            return false;
        }

        is >> input;
        if(input == ";") // broken / empty nonterminal
        {
            goto reading_nonterminal;
        }
        if(input != ":")
        {
            std::cerr << "expected : but got " << input << " after nonterminal " << nonterminal->Name << std::endl;
            return false;
        }

    reading_new_production:
        production = new Production();

    reading_production_part:
        is >> input;
        if(input == ";")
        {
            nonterminal->AddOrDeleteProduction(production);
            goto reading_nonterminal;
        }
        if(input == "|")
        {
            nonterminal->AddOrDeleteProduction(production);
            goto reading_new_production;
        }

        if(symbols.find(input) == symbols.end())
            AddNonTerminal(input);
        symbol = symbols[input];

        production->produced.push_back(symbol);
        goto reading_production_part;

    finished:
        is >> input;
        symbol = symbols[input];
        nonterminal = dynamic_cast<NonTerminal*>(symbol);
        if(nonterminal == NULL)
        {
            std::cerr << "start-symbol is " << input << " but must be a non-terminal" << std::endl;
            return false;
        }
        this->startsymbol = nonterminal;


    sanity_check:
        for(std::vector<NonTerminal*>::iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
            if((*i)->productions.size() <= 0)
                std::cerr << "Warning: Non-Terminal " << (*i)->Name << " has no productions. It was probably meant as terminal." << std::endl;
        return true;
}


template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::WriteToStream(std::ostream& os)
{
    for(typename std::vector<Terminal<token>*>::iterator t = terminals.begin(); t != terminals.end(); t++)
        os << (*t)->Name << " ";
    os << std::endl << "%%" << std::endl;

    for(std::vector<NonTerminal*>::iterator n = nonterminals.begin(); n != nonterminals.end(); n++)
    {
        os << (*n)->Name;
        bool first = true;
        for(std::vector<Production*>::iterator p = (*n)->productions.begin(); p != (*n)->productions.end(); p++)
        {
            os << (first?" :":" |");
            for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                os << " " << (*s)->Name;
            first = false;
        }
        os << " ;" << std::endl;
    }
    os << "%%" << std::endl << startsymbol->Name << std::endl;
}






template<class token> ContextFreeGrammars::Terminal<token>* ContextFreeGrammars::ContextFreeGrammar<token>::AddTerminal(token Token)
{
    std::stringstream s;
    s << Token;
    return AddTerminal(Token, s.str());
}
template<class token> ContextFreeGrammars::Terminal<token>* ContextFreeGrammars::ContextFreeGrammar<token>::AddTerminal(token Token, std::string Name)
{
    Terminal<token>* t = new Terminal<token>(Name, Token);
    terminals.push_back(t);
    symbols[Name] = t;
    return t;
}
template<class token> ContextFreeGrammars::NonTerminal* ContextFreeGrammars::ContextFreeGrammar<token>::AddNonTerminal(std::string Name)
{
    NonTerminal* nt = new NonTerminal(Name);
    nonterminals.push_back(nt);
    symbols[Name] = nt;
    return nt;
}
template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::RemoveSymbol(Symbol* symbol)
{
    bool productionErased = false;
    for(std::vector<NonTerminal*>::iterator n = nonterminals.begin(); n != nonterminals.end(); n += !productionErased)
    {
        productionErased = false;
        for(std::vector<Production*>::iterator p = (*n)->productions.begin(); (!productionErased) && p != (*n)->productions.end(); p++)
        {
            for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); (!productionErased) && s != (*p)->produced.end(); s++)
                if(*s == symbol)
                {
                    delete *p;
                    (*n)->productions.erase(p);
                    productionErased = true; // this aborts the outer loop (necessary because the iterator is invalidated)
                }
        }
    }

    symbols.erase(symbol->Name);
    std::vector<NonTerminal*>::iterator ntm = find(nonterminals.begin(), nonterminals.end(), symbol);
    if(ntm != nonterminals.end())
        nonterminals.erase(ntm);
    else
        terminals.erase(find(terminals.begin(), terminals.end(), symbol));
    delete symbol;
}






template<class token> std::vector<ContextFreeGrammars::NonTerminal*> ContextFreeGrammars::ContextFreeGrammar<token>::FindUnproductiveNonTerminals() const
{
    std::map<Symbol*, bool> productive;

    // all Terminals are "productive"
    for(typename std::vector<Terminal<token>*>::const_iterator t = terminals.begin(); t != terminals.end(); t++)
        productive[*t] = true;
    // non-terminals are unproductive until proven otherwise
    for(std::vector<NonTerminal*>::const_iterator n = nonterminals.begin(); n != nonterminals.end(); n++)
        productive[*n] = false;

    bool found_new = false;
    do
    {
        found_new = false;

        for(std::vector<NonTerminal*>::const_iterator n = nonterminals.begin(); n != nonterminals.end(); n++)
        {
            if(productive[*n]) // nonterminal is already marked as productive
                continue;

            for(std::vector<Production*>::iterator p = (*n)->productions.begin(); p != (*n)->productions.end(); p++)
            {
                bool productionproductive = true;
                for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                    if(!productive[*s])
                    {
                        productionproductive = false;
                        break;
                    }
                if(productionproductive)
                {
                    productive[*n] = true;
                    found_new = true;
                    break;
                }
            }
        }
    } while(found_new);


    std::vector<NonTerminal*> result;
    for(std::vector<NonTerminal*>::const_iterator n = nonterminals.begin(); n != nonterminals.end(); n++)
        if(!productive[*n])
            result.push_back(*n);
    return result;
}
template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::RemoveUnproductiveNonTerminals()
{
    std::vector<NonTerminal*> Unproductive = FindUnproductiveNonTerminals();
    for(std::vector<NonTerminal*>::iterator n = Unproductive.begin(); n != Unproductive.end(); n++)
        RemoveSymbol(*n);
}






template<class token> std::vector<ContextFreeGrammars::Symbol*> ContextFreeGrammars::ContextFreeGrammar<token>::FindUnreachableSymbols() const
{
    std::map<Symbol*, bool> reachable;
    for(std::map<std::string,Symbol*>::const_iterator i = symbols.begin(); i != symbols.end(); i++)
        reachable[i->second] = false;
    reachable[startsymbol] = true;

    std::queue<Symbol*> queue;
    for(queue.push(startsymbol); !queue.empty(); queue.pop())
    {
        Symbol* front = queue.front();
        NonTerminal* nFront = dynamic_cast<NonTerminal*>(front);
        if(nFront == NULL) // Terminal
            continue;

        for(std::vector<Production*>::iterator p = nFront->productions.begin(); p != nFront->productions.end(); p++)
        {
            for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
            {
                if(reachable[*s]) // already marked
                    continue;
                reachable[*s] = true;
                queue.push(*s);
            }
        }
    }

    std::vector<Symbol*> result;
    for(std::map<Symbol*,bool>::iterator i = reachable.begin(); i != reachable.end(); i++)
        if(!reachable[i->first])
            result.push_back(i->first);
    return result;
}
template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::RemoveUnreachableSymbols()
{
    std::vector<Symbol*> Unreachable = FindUnreachableSymbols();
    for(std::vector<Symbol*>::iterator s = Unreachable.begin(); s != Unreachable.end(); s++)
        RemoveSymbol(*s);
}






template<class token> std::set<const ContextFreeGrammars::NonTerminal*> ContextFreeGrammars::ContextFreeGrammar<token>::FindEpsilonProducers() const
{
    std::set<const NonTerminal*> result;

    // collect non-terminals A that have explicit  A -> epsilon  productions
    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
        for(std::vector<Production*>::const_iterator p = (*i)->productions.begin(); p != (*i)->productions.end(); p++)
            if((*p)->produced.size() <= 0)
                result.insert(*i);

    bool foundNew = result.size() > 0;
    while(foundNew)
    {
        foundNew = false;
        for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); !foundNew && i != nonterminals.end(); i++)
        {
            if(result.find(*i) != result.end()) // already recognized as epsilon-producer
                continue;

            for(std::vector<Production*>::const_iterator p = (*i)->productions.begin(); !foundNew && p != (*i)->productions.end(); p++)
            {
                bool producesEpsilon = true;
                for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); producesEpsilon && s != (*p)->produced.end(); s++)
                    if(result.find(dynamic_cast<NonTerminal*>(*s)) == result.end())
                        producesEpsilon = false; // part of the production is not (yet) an epsilon-producer

                if(producesEpsilon)
                {
                    result.insert(*i);
                    foundNew = true;
                }
            }
        }
    }

    return result;
}

template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::AddEpsilonCleanedProductions(ContextFreeGrammars::Production* prod, int position,
                             std::vector<ContextFreeGrammars::Symbol*>& stack, std::set<const ContextFreeGrammars::NonTerminal*>& epsProducers,
                             NonTerminal* nt, std::map<Symbol*, Symbol*>& correspondingsymbol)
{
    if(position >= prod->produced.size())
    {
        if(stack.size() > 0) // otherwise this stack corresponds to epsilon-production => eliminated
        {
            Production* newProd = new Production();
            newProd->produced = stack;
            nt->AddOrDeleteProduction(newProd);
        }
        return;
    }

    NonTerminal* prodsymnt = dynamic_cast<NonTerminal*>(prod->produced[position]);

    stack.push_back(correspondingsymbol[prod->produced[position]]);
    AddEpsilonCleanedProductions(prod, position+1, stack, epsProducers, nt, correspondingsymbol);
    stack.pop_back();

    if(prodsymnt != NULL && epsProducers.find(prodsymnt) != epsProducers.end())
        AddEpsilonCleanedProductions(prod, position+1, stack, epsProducers, nt, correspondingsymbol);
}
template<class token> ContextFreeGrammars::ContextFreeGrammar<token> ContextFreeGrammars::ContextFreeGrammar<token>::EpsilonElimination() const
{
    std::set<const ContextFreeGrammars::NonTerminal*> epsProducers = FindEpsilonProducers();
    if(epsProducers.size() <= 0)
        return *this;
    ContextFreeGrammar result;
    std::map<Symbol*, Symbol*> correspondingsymbol;

    // copy terminals
    for(typename std::vector<Terminal<token>*>::const_iterator i = terminals.begin(); i != terminals.end(); i++)
        correspondingsymbol[*i] = result.AddTerminal((*i)->Token, (*i)->Name);

    // copy nonterminals
    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
        correspondingsymbol[*i] = result.AddNonTerminal((*i)->Name);

    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
    {
        NonTerminal* nt = static_cast<NonTerminal*>(correspondingsymbol[*i]);
        std::vector<Symbol*> stack;
        for(std::vector<Production*>::const_iterator p = (*i)->productions.begin(); p != (*i)->productions.end(); p++)
            result.AddEpsilonCleanedProductions(*p, 0, stack, epsProducers, nt, correspondingsymbol);
    }

    if(epsProducers.find(startsymbol) != epsProducers.end())
    {
        NonTerminal *s0 = result.AddNonTerminal(startsymbol->Name + "_0");

        Production *s0sProduction = new Production();
        s0sProduction->produced.push_back(correspondingsymbol[startsymbol]);
        s0->AddOrDeleteProduction(s0sProduction);

        Production *s0epsProduction = new Production();
        s0->AddOrDeleteProduction(s0epsProduction);

        result.startsymbol = s0;
    }
    else
    {
        result.startsymbol = static_cast<NonTerminal*>(correspondingsymbol[startsymbol]);
    }

    return result;
}






template<class token> ContextFreeGrammars::ContextFreeGrammar<token> ContextFreeGrammars::ContextFreeGrammar<token>::UnitProductionElimination() const
{
    ContextFreeGrammar<token> result;
    std::map<Symbol*, Symbol*> correspondingsymbol;
    std::map<Symbol*, std::set<NonTerminal*> > reachable;

    // copy terminals
    for(typename std::vector<Terminal<token>*>::const_iterator i = terminals.begin(); i != terminals.end(); i++)
        correspondingsymbol[*i] = result.AddTerminal((*i)->Token, (*i)->Name);

    // copy nonterminals
    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
        correspondingsymbol[*i] = result.AddNonTerminal((*i)->Name);

    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
    {
        reachable[*i].insert(*i);
        std::queue<Symbol*> queue;
        for(queue.push(*i); !queue.empty(); queue.pop())
        {
            NonTerminal* front = dynamic_cast<NonTerminal*>(queue.front());

            for(std::vector<Production*>::iterator p = front->productions.begin(); p != front->productions.end(); p++)
            {
                if((*p)->produced.size() != 1) // not a unit production
                    continue;
                Symbol *prod = *((*p)->produced.begin());
                NonTerminal *nprod = dynamic_cast<NonTerminal*>(prod);
                if(nprod == NULL) // not a non-terminal
                    continue;
                if(reachable[*i].find(nprod) != reachable[*i].end()) // already marked
                    continue;
                reachable[*i].insert(nprod);
                queue.push(nprod);
            }
        }
    }

    // copy (non-unit) productions
    for(std::vector<NonTerminal*>::const_iterator i = nonterminals.begin(); i != nonterminals.end(); i++)
    {
        for(std::set<NonTerminal*>::const_iterator j = reachable[*i].begin(); j != reachable[*i].end(); j++)
        {
            for(std::vector<Production*>::const_iterator p = (*j)->productions.begin(); p != (*j)->productions.end(); p++)
            {
                if((*p)->produced.size() == 1) // unit productions are not copied
                    if( dynamic_cast<NonTerminal*>(*((*p)->produced.begin())) != NULL )
                        continue;

                Production *prodCopy = new Production();
                for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                    prodCopy->produced.push_back(correspondingsymbol[*s]);
                NonTerminal* corresponding = static_cast<NonTerminal*>(correspondingsymbol[*i]);
                corresponding->AddOrDeleteProduction(prodCopy);
            }
        }
    }

    result.startsymbol = static_cast<NonTerminal*>(correspondingsymbol[startsymbol]);
    return result;
}






template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::Produce(int depth, const std::vector<const Symbol*>& stack, void callback(const std::vector<const Symbol*>&)) const
{
    if(depth < 0)
        return;

    bool isTerminalWord = true;
    std::vector<const Symbol*> newstack;
    for(std::vector<const Symbol*>::const_iterator i = stack.begin(); i != stack.end(); i++)
    {
        const NonTerminal* n = dynamic_cast<const NonTerminal*>(*i);
        if(n == NULL) // terminals are not
            continue;
        isTerminalWord = false;

        for(std::vector<Production*>::const_iterator p = n->productions.begin(); p != n->productions.end(); p++)
        {
            newstack.clear();
            for(std::vector<const Symbol*>::const_iterator k = stack.begin(); k != i; k++) // front of old stack
                newstack.push_back(*k);
            for(std::vector<Symbol*>::const_iterator k = (*p)->produced.begin(); k != (*p)->produced.end(); k++) // replacement from current production
                newstack.push_back(*k);
            for(std::vector<const Symbol*>::const_iterator k = i+1; k != stack.end(); k++) // tail of old stack
                newstack.push_back(*k);
            Produce(depth-1, newstack, callback);
        }
    }

    if(isTerminalWord)
        callback(stack);
}
template<class token> void ContextFreeGrammars::ContextFreeGrammar<token>::Produce(int depth, void callback(const std::vector<const ContextFreeGrammars::Symbol*>&)) const
{
    std::vector<const Symbol*> stack(1, startsymbol);
    Produce(depth, stack, callback);
}






template<class token> ContextFreeGrammars::ContextFreeGrammar<token> ContextFreeGrammars::ContextFreeGrammar<token>::ChomskyNormalForm() const
{
    const ContextFreeGrammar<token> *temp0 = this;
/*
    temp0 = temp0.EpsilonElimination();
    temp0 = temp0.UnitProductionElimination();
*/
    ContextFreeGrammar<token> temp1;
    std::map<Symbol*, NonTerminal*> correspondingsymbol;

    // copy nonterminals
    for(std::vector<NonTerminal*>::const_iterator i = temp0->nonterminals.begin(); i != temp0->nonterminals.end(); i++)
        correspondingsymbol[*i] = temp1.AddNonTerminal((*i)->Name);

    // create R_a -> a  productions
    for(typename std::vector<Terminal<token>*>::const_iterator i = temp0->terminals.begin(); i != temp0->terminals.end(); i++)
    {
        Terminal<token>* t = temp1.AddTerminal((*i)->Token, (*i)->Name);
        NonTerminal* nt = temp1.AddNonTerminal("R_{" + (*i)->Name + "}");
        Production* p = new Production();
        p->produced.push_back(t);
        nt->AddOrDeleteProduction(p);
        correspondingsymbol[*i] = nt;
    }

    // break overlong productions apart
    for(std::vector<NonTerminal*>::const_iterator i = temp0->nonterminals.begin(); i != temp0->nonterminals.end(); i++)
    {
        NonTerminal* nt = correspondingsymbol[*i];
        int pidx = 1;

        for(std::vector<Production*>::iterator p = (*i)->productions.begin(); p != (*i)->productions.end(); p++, pidx++)
        {
            if((*p)->produced.size() >= 2)
            {
                // A -> X1 X2 X3 X4 becomes  A -> X1 A1
                //                          A1 -> X2 A2
                //                          A2 -> X3 X4

                std::string prefix = nt->Name + "_{" + std::to_string(pidx) + ",";
                NonTerminal *lastnonterminal = nt;

                for(int j = 1; j < (*p)->produced.size()-1; ++j)
                {
                    // create next nonterminal Aj
                    NonTerminal *nj = temp1.AddNonTerminal(prefix + std::to_string(j) + "}");

                    // create the production lastnonterminal -> Xj Aj
                    Production *pj = new Production();
                    pj->produced.push_back(correspondingsymbol[(*p)->produced[j-1]]);
                    pj->produced.push_back(nj);

                    // add the production to lastnonterminal
                    lastnonterminal->AddOrDeleteProduction(pj);
                    lastnonterminal = nj;
                }

                // create An-2 -> Xn-1 Xn
                int last = (*p)->produced.size()-1;
                Production *plast = new Production();
                plast->produced.push_back(correspondingsymbol[(*p)->produced[last-1]]);
                plast->produced.push_back(correspondingsymbol[(*p)->produced[last]]);
                lastnonterminal->AddOrDeleteProduction(plast);
            }
            else
            {
                Production *prodCopy = new Production();
                for(std::vector<Symbol*>::iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                    prodCopy->produced.push_back(correspondingsymbol[*s]);
                nt->AddOrDeleteProduction(prodCopy);
            }
        }
    }

    temp1.startsymbol = correspondingsymbol[temp0->startsymbol];

    ContextFreeGrammar<token> temp2 = temp1.EpsilonElimination();
    ContextFreeGrammar<token> temp3 = temp2.UnitProductionElimination();
    temp3.RemoveUnproductiveNonTerminals();
    temp3.RemoveUnreachableSymbols();
    return temp3;
}




template<class token> bool ContextFreeGrammars::ContextFreeGrammar<token>::Producable(const std::vector<std::string>& w) const
{
    std::vector<Symbol*> s;
    for(std::vector<std::string>::const_iterator i = w.begin(); i != w.end(); i++)
    {
        std::map<std::string, Symbol*>::const_iterator it = symbols.find(*i);
        Symbol* is = it->second;
        s.push_back(is);
    }
    return Producable(s);
}

template<class token> bool ContextFreeGrammars::ContextFreeGrammar<token>::Producable(const std::vector<Symbol*>& w) const
{
    ContextFreeGrammar<token> cnf = ChomskyNormalForm();

    std::vector<std::vector<std::set<NonTerminal*> > > Table;


}

#endif
